@@ -5,7 +5,7 @@ from itertools import cycle
 from pathlib import Path
 from re import match
 from typing import Dict, Generator, List
-from zipfile import ZipFile
+from zipfile import BadZipFile, ZipFile
 
 import psutil
 import requests
@@ -13,8 +13,8 @@ from attrs import define, field
 from bs4 import BeautifulSoup
 from tqdm.auto import tqdm
 
+from .data import ENUMERATED_TABLES, MMSDM_ARCHIVE_URL, PREDISP_ALL_DATA, USER_AGENTS
 from .data_handlers import clean_forecast_csv
-from .downloader_helpers.data import MMSDM_ARCHIVE_URL, PREDISP_ALL_DATA, USER_AGENTS
 from .query import (
     Query,
     _construct_sqlloader_filename,
@@ -26,6 +26,7 @@ logger = logging.getLogger(__name__)
 
 
 def _validate_forecast_type(forecast_type: str):
+    """Check user-supplied forecast type is valid"""
     valid_types = ("P5MIN", "PREDISPATCH", "PDPASA", "STPASA", "MTPASA")
     if forecast_type not in valid_types:
         raise ValueError(f"Forecast type should be one of {valid_types}")
@@ -129,7 +130,7 @@ def _construct_yearmonth_url(
     Args:
         year: Year
         month: Month
-        forecast_type: AEMO forecast types (`P5MIN`, `PREDISPATCH`, `STPASA`, `MTPASA`)
+        forecast_type: One of :data:`nemseer.forecast_types`
         all_data (optional): Default False. Points to `ALL_DATA` folder for PD
     Returns:
         Constructed URL as a string.
@@ -157,7 +158,7 @@ def _construct_sqlloader_forecastdata_url(
     Args:
         year: Year
         month: Month
-        forecast_type: AEMO forecast types (`P5MIN`, `PREDISPATCH`, `STPASA`, `MTPASA`)
+        forecast_type: One of :data:`nemseer.forecast_types`
     Returns:
         URL to zip file
     """
@@ -183,7 +184,7 @@ def _get_captured_group_from_links(url: str, regex: str) -> List[str]:
     Args:
         year: Year
         month: Month
-        forecast_type: AEMO forecast types (`P5MIN`, `PREDISPATCH`, `STPASA`, `MTPASA`)
+        forecast_type: One of :data:`nemseer.forecast_types`
         regex: Regular expression pattern, with one group capture
     Returns:
         A list of unique captured groups (one for each link on the page of tables)
@@ -214,7 +215,8 @@ def get_sqlloader_forecast_tables(
     Args:
         year: Year
         month: Month
-        forecast_type: AEMO forecast types (`P5MIN`, `PREDISPATCH`, `STPASA`, `MTPASA`)
+        forecast_type: One of :data:`nemseer.forecast_types`
+
     Returns:
         List of tables associated with that forecast type for that period
     """
@@ -286,23 +288,37 @@ def get_sqlloader_years_and_months() -> Dict[int, List[int]]:
 
 
 def get_unzipped_csv(url: str, raw_cache: Path) -> None:
-    """Unzipped (single) csv file downloaded from `url` to `raw_cache`
+    """Unzipped (single) csv file downloaded from `url` to :attr:`raw_cache`
 
     1. Downloads zip file in chunks to limit memory use and enable progress bar
     2. Validates that the zip contains a single file that has the same name as the zip
+    3. If the zip file is invalid, writes the file stub to `.invalid_aemo_files.txt`
 
     Args:
         url: URL of zip
         raw_cache: Path to save zip
     Returns:
-        None. Extracts csvs to `raw_cache`.
+        None. Extracts csvs to :attr:`raw_cache`.
     """
+
+    def _invalid_zip_to_file(invalid_files: Path, filename: str) -> None:
+        """Ensure that any invalid file is noted in the `invalid_files` text file"""
+        with open(invalid_files, "a+") as f:
+            f.seek(0)
+            existing = [line.strip() for line in f.readlines()]
+            if filename in existing:
+                pass
+            else:
+                f.write(f"{filename}\n")
+        return None
+
     file_name = Path(url).name
     header = _build_nemweb_get_header(next(_build_useragent_generator(1)))
     file_path = raw_cache / Path(file_name)
-    with requests.get(url, headers=header, stream=True) as r:
-        total_length = int(r.headers.get("Content-Length", 0))
-        with tqdm.wrapattr(r.raw, "read", desc=file_name, total=total_length) as raw:
+    with requests.get(url, headers=header, stream=True) as resp:
+        total_length = int(resp.headers.get("Content-Length", 0))
+        resp.raise_for_status()
+        with tqdm.wrapattr(resp.raw, "read", desc=file_name, total=total_length) as raw:
             with open(file_path, "wb") as fout:
                 shutil.copyfileobj(raw, fout)
     z = ZipFile(file_path)
@@ -312,20 +328,25 @@ def get_unzipped_csv(url: str, raw_cache: Path) -> None:
         and (fn := match("(.*).[cC][sS][vV]", csvfn.pop()))
         and (fn.group(1) == zfn.group(1))
     ):
-        z.extractall(raw_cache)
-        z.close()
+        try:
+            z.extractall(raw_cache)
+            z.close()
+        except BadZipFile:
+            logging.error(f"{z.testzip()} invalid or corrupted")
+            invalid_files = raw_cache / Path(".invalid_aemo_files.txt")
+            _invalid_zip_to_file(invalid_files, fn.group(1))
         Path(file_path).unlink()
     else:
         raise ValueError(f"Unexpected contents in zipfile from {url}")
 
 
-def _validate_tables_on_forecast_start(instance, attribute, value) -> None:
+def _validate_tables_on_run_start(instance, attribute, value) -> None:
     """Validates tables for the provided forecast type.
 
     Checks user-supplied tables against tables available in MMS Historical
-    Data SQLLoader for the month and year of forecast_start.
+    Data SQLLoader for the month and year of run_start.
     """
-    start_dt = instance.forecast_start
+    start_dt = instance.run_start
     actual_tables = get_sqlloader_forecast_tables(
         start_dt.year, start_dt.month, instance.forecast_type, actual=True
     )
@@ -342,65 +363,59 @@ def _validate_tables_on_forecast_start(instance, attribute, value) -> None:
 
 @define(kw_only=True)
 class ForecastTypeDownloader:
-    forecast_start: datetime
-    forecast_end: datetime
+    run_start: datetime
+    run_end: datetime
     forecast_type: str
-    tables: List[str] = field(validator=_validate_tables_on_forecast_start)
+    tables: List[str] = field(validator=_validate_tables_on_run_start)
     raw_cache: Path
 
     @classmethod
     def from_Query(cls, query: Query):
-        """Constructor method for ForecastTypeDownquery from Query."""
+        """Constructor method for :class:`ForecastTypeDownloader` from
+        :class:`Query <nemseer.query.Query>`"""
         tables = query.tables
-        enumerated_cases = {
-            "P5MIN": [("CONSTRAINTSOLUTION", 4)],
-            "PREDISPATCH": [("CONSTRAINT", 2), ("LOAD", 2)],
-        }
-        for ftype in enumerated_cases:
+        for ftype in ENUMERATED_TABLES:
             if query.forecast_type == ftype:
-                for table, enumerate_to in enumerated_cases[ftype]:
+                for table, enumerate_to in ENUMERATED_TABLES[ftype]:
                     if table in tables:
                         tables = _enumerate_tables(tables, table, enumerate_to)
 
         return cls(
-            forecast_start=query.forecast_start,
-            forecast_end=query.forecast_end,
+            run_start=query.run_start,
+            run_end=query.run_end,
             forecast_type=query.forecast_type,
             tables=tables,
             raw_cache=query.raw_cache,
         )
 
     def download_csv(self) -> None:
-        """Downloads and unzips zip files given query loaded into ForecastTypeDownloader
+        """Downloads and unzips zip files given query loaded into
+        :class:`ForecastTypeDownloader`
 
         This method will only download and unzip the relevant zip/csv if the
-        corresponding `.parquet` file is not located in the specified `raw_cache`.
+        corresponding `.parquet` file is not located in the specified :attr:`raw_cache`.
         """
-        yearmonths, fnames = generate_sqlloader_filenames(
-            self.forecast_start, self.forecast_end, self.forecast_type, self.tables
+        filename_data = generate_sqlloader_filenames(
+            self.run_start, self.run_end, self.forecast_type, self.tables
         )
-        for ((year, month), fname) in zip(yearmonths, fnames):
-            if raw_table := match(
-                f"^PUBLIC_DVD_{self.forecast_type}(.*)_[0-9]*$", fname
-            ):
-                table = raw_table.group(1).lstrip("_")
-                if (self.raw_cache / Path(fname + ".parquet")).exists():
-                    logging.info(f"{table} for {month}/{year} in raw_cache")
-                    continue
-                else:
-                    url = _construct_sqlloader_forecastdata_url(
-                        year, month, self.forecast_type, table
-                    )
-                    logger.info(f"Downloading and unzipping {table} for {month}/{year}")
-                    get_unzipped_csv(url, self.raw_cache)
+        for metadata in filename_data.keys():
+            fname = filename_data[metadata]
+            (year, month, table) = metadata
+            if (self.raw_cache / Path(fname + ".parquet")).exists():
+                logging.info(f"{table} for {month}/{year} in raw_cache")
+                continue
             else:
-                raise ValueError(f"Invalid file name: {fname}")
+                url = _construct_sqlloader_forecastdata_url(
+                    year, month, self.forecast_type, table
+                )
+                logger.info(f"Downloading and unzipping {table} for {month}/{year}")
+                get_unzipped_csv(url, self.raw_cache)
 
     def convert_to_parquet(self, keep_csv=False) -> None:
-        """Converts all CSVs in the `raw_cache` to parquet
+        """Converts all CSVs in the :attr:`raw_cache` to parquet
 
-        Logs a warning if the filesize is greater than half of available memory.
-        pandas DataFrames consume more than the file size in memory.
+        Logs a warning if the filesize is greater than half of available memory as
+        :class:`pandas.DataFrame` consumes more than the file size in memory.
         """
         csvs = list(Path(self.raw_cache).glob("*.[Cc][Ss][Vv]"))
         for csv in csvs:
