@@ -6,6 +6,8 @@ from typing import Dict, List, Optional, Tuple, Union
 from attrs import converters, define, field, validators
 from dateutil import rrule
 
+from .data import ENUMERATED_TABLES
+
 logger = logging.getLogger(__name__)
 
 
@@ -16,12 +18,16 @@ def _dt_converter(value: str) -> datetime:
         value: String with format %Y/%m/%d %H:%M
     Returns:
         Datetime object
+    Raises:
+        ValueError: If provided datetime string is invalid
     """
     try:
         format = "%Y/%m/%d %H:%M"
         return datetime.strptime(value, format)
     except ValueError:
-        raise ValueError("Datetime should be provided as follows: yyyy/mm/dd HH:MM")
+        raise ValueError(
+            "Datetime invalid. Datetime should be provided as follows: yyyy/mm/dd HH:MM"
+        )
 
 
 def _tablestr_converter(value: Union[str, List[str]]) -> List[str]:
@@ -38,12 +44,12 @@ def _tablestr_converter(value: Union[str, List[str]]) -> List[str]:
         return list(value)
 
 
-def _validate_forecast_chronology(instance, attribute, value):
-    """Validates forecast_start against forecast_end"""
-    if value > instance.forecast_end:
+def _validate_run_chronology(instance, attribute, value):
+    """Validates run_start against run_end"""
+    if value > instance.run_end:
         raise ValueError(
             "Forecast end datetime must be greater than or equal to"
-            + " forecast start datetime."
+            + " run start datetime."
         )
 
 
@@ -57,10 +63,10 @@ def _validate_forecasted_chronology(instance, attribute, value):
 
 
 def _validate_relative_chronology(instance, attribute, value) -> None:
-    """Validates forecast_start against forecasted_start"""
+    """Validates run_start against forecasted_start"""
     if value > instance.forecasted_start:
         raise ValueError(
-            "Forecasted start datetime should be at or after forecast start datetime."
+            "Forecasted start datetime should be at or after run start datetime."
         )
 
 
@@ -72,7 +78,7 @@ def _validate_path(instance, attribute, value) -> None:
 
 
 def _validate_raw_not_processed(instance, attribute, value) -> None:
-    """Check that `raw_cache` and `processed_cache` are distinct."""
+    """Check that :attr:`raw_cache` and :attr:`processed_cache` are distinct."""
     if instance.processed_cache:
         if value.absolute() == instance.processed_cache.absolute():
             raise ValueError(
@@ -107,7 +113,7 @@ def _construct_sqlloader_filename(
     Args:
         year: Year
         month: Month
-        forecast_type: `P5MIN`, `PREDISPATCH`, `PDPASA`, `STPASA` or `MTPASA`
+        forecast_type: One of :data:`nemseer.forecast_types`
         table: The name of the table required
     Returns:
         Filename string without file type
@@ -122,63 +128,68 @@ def _construct_sqlloader_filename(
 
 
 def generate_sqlloader_filenames(
-    forecast_start: datetime,
-    forecast_end: datetime,
+    run_start: datetime,
+    run_end: datetime,
     forecast_type: str,
     tables: List[str],
-) -> Tuple[List[Tuple[int, int]], List[str]]:
-    """Generates a list of MMSDM Historical Data SQLLoader file names based on query
+) -> Dict[Tuple[int, int, str], str]:
+    """Generates MMSDM Historical Data SQLLoader file names based on provided query data
+
+    Returns a tuple of query metadata (`table`, `year`, `month`) mapped to each filename
 
     Args:
-        forecast_start: Forecasts made at or after this datetime are queried.
-        forecast_end: Forecasts made before or at this datetime are queried.
-        forecast_type: `MTPASA`, `STPASA`, `PDPASA`, `PREDISPATCH` or `P5MIN`.
+        run_start: Forecast runs at or after this datetime are queried.
+        run_end: Forecast runs before or at this datetime are queried.
+        forecast_type: One of :data:`nemseer.forecast_types`
         tables: Table or tables required, provided as a List.
     Returns:
-        A list of year-month tuples and corresponding format-agnostic
-        (SQLLOader) filenames
+        A tuple of query metadata (`table`, `year`, `month`) mapped to each
+        format-agnostic (SQLLOader) filename
     """
-    intervening_dates = rrule.rrule(
-        rrule.MONTHLY, dtstart=forecast_start, until=forecast_end
-    )
-    year_months = []
-    fnames = []
-    if "CONSTRAINTSOLUTION" in tables and forecast_type == "P5MIN":
-        tables = _enumerate_tables(tables, "CONSTRAINTSOLUTION", 4)
+    intervening_dates = rrule.rrule(rrule.MONTHLY, dtstart=run_start, until=run_end)
+    filename_data = {}
+    for ftype in ENUMERATED_TABLES:
+        if forecast_type == ftype:
+            for table, enumerate_to in ENUMERATED_TABLES[ftype]:
+                if table in tables:
+                    tables = _enumerate_tables(tables, table, enumerate_to)
     for table in tables:
         for date in intervening_dates:
             (year, month) = (date.year, date.month)
             fname = _construct_sqlloader_filename(year, month, forecast_type, table)
-            fnames.append(fname)
-            year_months.append((year, month))
-    return year_months, fnames
+            filename_data[(year, month, table)] = fname
+    return filename_data
 
 
 @define
 class Query:
-    """`Query` validates user inputs and dispatches data fetchers.
+    """`Query` validates user inputs and dispatches data downloaders and compilers
 
-    Construct `Query` using the `Query.initialise()` constructor. This
+    Construct :class:`Query` using the :meth:`Query.initialise()` constructor. This
     ensures query metadata is constructed approriately.
 
     Query:
 
     - Validates user input data
-        - Checks datetime are yyyy/mm/dd HH:MM
+        - Checks datetime fit `yyyy/mm/dd HH:MM` format
         - Checks datetime chronology (e.g. end is after start)
+        - Checks requested datetimes are valid for each `forecast_type`
         - Validates `forecast_type`
-        - *Validates user-supplied tables against what is available on NEMWeb*
-    - Retains query metadata (via constructor class method `initialise`)
-    - Can dispatch various Managers and Downloaders
+        - Validates user-requested tables against what is available on NEMWeb
+    - Retains query metadata (via constructor class method
+      :meth:`nemseer.query.Query.initialise`)
+    - Can dispatch
+      :class:`ForecastTypeDownloader <nemseer.downloader.ForecastTypeDownloader>` and
+      :class:`DataCompiler <nemseer.data_compilers.DataCompiler>`
 
     Attributes:
-        forecast_start: Forecasts made at or after this datetime are queried.
-        forecast_end: Forecasts made before or at this datetime are queried.
+        run_start: Forecast runs at or after this datetime are queried.
+        run_end: Forecast runs before or at this datetime are queried.
         forecasted_start: Forecasts pertaining to times at or after this
             datetime are retained.
         forecasted_end: Forecasts pertaining to times before or at this
             datetime are retaned.
-        forecast_type: `MTPASA`, `STPASA`, `PDPASA`, `PREDISPATCH` or `P5MIN`.
+        forecast_type: One of :data:`nemseer.forecast_types`
         tables: Table or tables required. A single table can be supplied as
             a string. Multiple tables can be supplied as a list of strings.
         metadata: Metadata dictionary. Constructed by `Query.initialise()`.
@@ -188,11 +199,11 @@ class Query:
 
     """
 
-    forecast_start: datetime = field(
+    run_start: datetime = field(
         converter=_dt_converter,
-        validator=[_validate_forecast_chronology, _validate_relative_chronology],
+        validator=[_validate_run_chronology, _validate_relative_chronology],
     )
-    forecast_end: datetime = field(converter=_dt_converter)
+    run_end: datetime = field(converter=_dt_converter)
     forecasted_start: datetime = field(
         converter=_dt_converter, validator=[_validate_forecasted_chronology]
     )
@@ -218,8 +229,8 @@ class Query:
     @classmethod
     def initialise(
         cls,
-        forecast_start: str,
-        forecast_end: str,
+        run_start: str,
+        run_end: str,
         forecasted_start: str,
         forecasted_end: str,
         forecast_type: str,
@@ -227,38 +238,40 @@ class Query:
         raw_cache: str,
         processed_cache: Optional[str] = None,
     ) -> "Query":
-        """Constructor method for Query. Assembles query metatdata."""
+        """Constructor method for :class:`Query`. Assembles query metatdata."""
         metadata = {
-            "forecast_start": forecast_start,
-            "forecast_end": forecast_end,
+            "run_start": run_start,
+            "run_end": run_end,
             "forecasted_start": forecasted_start,
             "forecasted_end": forecasted_end,
             "forecast_type": forecast_type,
             "tables": tables,
         }
         return cls(
-            forecast_start=forecast_start,
-            forecast_end=forecast_end,
-            forecasted_start=forecasted_start,
-            forecasted_end=forecasted_end,
-            forecast_type=forecast_type,
-            tables=tables,
-            metadata=metadata,
-            raw_cache=raw_cache,
-            processed_cache=processed_cache,
+            run_start=run_start,  # type: ignore
+            run_end=run_end,  # type: ignore
+            forecasted_start=forecasted_start,  # type: ignore
+            forecasted_end=forecasted_end,  # type: ignore
+            forecast_type=forecast_type,  # type: ignore
+            tables=tables,  # type: ignore
+            metadata=metadata,  # type: ignore
+            raw_cache=raw_cache,  # type: ignore
+            processed_cache=processed_cache,  # type: ignore
         )
 
     def check_data_in_cache(self) -> bool:
-        """Checks whether *all* requested data is already in the `raw_cache` as parquet
+        """Checks whether *all* requested data is already in the :attr:`raw_cache` as
+        parquet
 
-        `downloader` methods handle partial `raw_cache` completeness
+        :meth:`nemseer.downloader.ForecastTypeDownloader.download_csv()`
+        handles partial :attr:`raw_cache` completeness
 
-        If all requested data is already in the `raw_cache` as parquet, returns True
-        Otherwise returns False.
+        If all requested data is already in the :attr:`raw_cache` as parquet,
+        returns True. Otherwise returns False.
         """
-        _, fnames = generate_sqlloader_filenames(
-            self.forecast_start, self.forecast_end, self.forecast_type, self.tables
-        )
+        fnames = generate_sqlloader_filenames(
+            self.run_start, self.run_end, self.forecast_type, self.tables
+        ).values()
         check = [
             (self.raw_cache / Path(fname + ".parquet")).exists() for fname in fnames
         ]
